/**
 * ====================================================================
 * asynccallhandler.cpp
 *
 * Copyright (c) 2006 Nokia Corporation
 * Copyright (c) 2008 Christophe Berger <chris@berger.cx>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

#include <e32base.h>
#include "AsyncCallHandler.h"

void CAsyncCallHandler::ConstructL()
{ 
  iWait = new (ELeave) CActiveSchedulerWait;
  inProgress = None;
  CActiveScheduler::Add(this);
};

CAsyncCallHandler::CAsyncCallHandler(CTelephony& telephony, RFileLogger& log):CActive(CActive::EPriorityStandard),
	iTelephony(telephony), iLog(log)
{
};

CAsyncCallHandler* CAsyncCallHandler::NewL(CTelephony& telephony, RFileLogger& log)
{
  CAsyncCallHandler* self = new (ELeave) CAsyncCallHandler(telephony, log);
  CleanupStack::PushL(self);
  self->ConstructL();
  CleanupStack::Pop(self);
  self->iLog.Write(_L("CAsyncCallHandler::NewL"));
  return self;
};

void CAsyncCallHandler::DoCancel()
{
  this->iLog.WriteFormat(_L("CAsyncCallHandler::DoCancel: %i"), this->inProgress);
  if (this->inProgress == NetworkInfo) {
	this->iTelephony.CancelAsync(CTelephony::EGetCurrentNetworkInfoCancel);
	iLog.Write(_L("CAsyncCallHandler::DoCancel: NetworkInfo"));
  }
  else {
    if (this->inProgress == RegistrationInfo) {
	  this->iTelephony.CancelAsync(CTelephony::EGetNetworkRegistrationStatusCancel);
	  iLog.Write(_L("CAsyncCallHandler::DoCancel: RegistrationInfo"));
	}
    else {
	  if (this->inProgress == SubscriberInfo) {
	    this->iTelephony.CancelAsync(CTelephony::EGetSubscriberIdCancel);
		iLog.Write(_L("CAsyncCallHandler::DoCancel: EGetSubscriberId"));
	  }
	}
  }
  //else
  //	iLog :: error
  this->inProgress = None;
};

CAsyncCallHandler::~CAsyncCallHandler()
{
  Cancel();
  delete iWait;
};

void CAsyncCallHandler::RunL()
{
 this->iWait->AsyncStop();
};

void CAsyncCallHandler::MakeAsyncCall(CTelephony::TNetworkInfoV1Pckg& networkInfo,TRequestStatus& status)
{
  if (this->inProgress != None)
  {
	status = KErrInUse;
	return;
  }
  this->inProgress = NetworkInfo;
  this->iTelephony.GetCurrentNetworkInfo(iStatus, networkInfo);
  iLog.Write(_L("CAsyncCallHandler::MakeAsyncCall"));
  this->SetActive();
  this->iWait->Start();
  status = this->iStatus;
  this->inProgress = None;
}

void CAsyncCallHandler::MakeAsyncCall_Reg(CTelephony::TNetworkRegistrationV1Pckg& iRegInfo, TRequestStatus& status)
{
  if (this->inProgress != None)
  {
	status = KErrInUse;
	return;
  }
  this->inProgress = RegistrationInfo;
  this->iTelephony.GetNetworkRegistrationStatus(status, iRegInfo);
  iLog.Write(_L("CAsyncCallHandler::MakeAsyncCall_Reg"));
  this->SetActive();
  this->iWait->Start();
  status = this->iStatus;
  this->inProgress = None;
}

void CAsyncCallHandler::MakeAsyncCall_IMSI(CTelephony::TSubscriberIdV1Pckg& iSubsInfo, TRequestStatus& status)
{
  if (this->inProgress != None)
  {
	status = KErrInUse;
	return;
  }
  this->inProgress = SubscriberInfo;
  this->iTelephony.GetSubscriberId(status, iSubsInfo);
  iLog.Write(_L("CAsyncCallHandler::MakeAsyncCall_IMSI"));
  this->SetActive();
  this->iWait->Start();
  status = this->iStatus;
  this->inProgress = None;
}
