/**
 * ====================================================================
 *  extendedlocation.cpp
 *
 *  Python API to location information.
 *
 * Copyright (c) 2005 - 2007 Nokia Corporation
 * Copyright (c) 2007 Christophe Berger, LIP6 <christophe.berger@lip6.fr>
 * Copyright (c) 2008 Christophe Berger <chris@berger.cx>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

#include "Python.h"
#include "symbian_python_ext_util.h"

#include <eikenv.h>
#include <e32std.h>
#include <flogger.h>

#ifndef EKA2
#include <etelbgsm.h>
#else
#include <etel.h>
#include <etel3rdparty.h>
#include <e32cmn.h>
#include "asynccallhandler.h"
#endif

#ifndef EKA2
_LIT (KTsyName, "phonetsy.tsy");

extern "C" PyObject *
get_location(PyObject* /*self*/)
{
  TInt error = KErrNone;

  TInt enumphone = 1;
  RTelServer   server;
  RBasicGsmPhone phone;
  RTelServer::TPhoneInfo info;
  MBasicGsmPhoneNetwork::TCurrentNetworkInfo NetworkInfo;

  error = server.Connect();
  if (error != KErrNone)
    return SPyErr_SetFromSymbianOSErr(error);

  error = server.LoadPhoneModule(KTsyName);
  if (error != KErrNone)
    return SPyErr_SetFromSymbianOSErr(error);

  error = server.EnumeratePhones(enumphone);
  if (error != KErrNone)
    return SPyErr_SetFromSymbianOSErr(error);

  if (enumphone < 1)
    return SPyErr_SetFromSymbianOSErr(KErrNotFound);

  error = server.GetPhoneInfo(0, info);
  if (error != KErrNone)
    return SPyErr_SetFromSymbianOSErr(error);

  error = phone.Open(server, info.iName);
  if (error != KErrNone)
    return SPyErr_SetFromSymbianOSErr(error);

  error = phone.GetCurrentNetworkInfo(NetworkInfo);
  if (error != KErrNone)
    return SPyErr_SetFromSymbianOSErr(error);
           
  phone.Close();
  server.Close();

  return Py_BuildValue("(iiii)", NetworkInfo.iNetworkInfo.iId.iMCC,
           NetworkInfo.iNetworkInfo.iId.iMNC,
           NetworkInfo.iLocationAreaCode,
           NetworkInfo.iCellId);
}


#else

extern "C"
void location_mod_cleanup();

class TStaticData {
public:
  CTelephony *telephony;
  RFileLogger iLog;
};

extern "C" PyObject *
getVersion(PyObject* /*self*/)
{
	return Py_BuildValue("s", "elocation v.0.2.7_d");
	// _d = debug with floger
}

TStaticData* GetTelephony()
{
  if (Dll::Tls())
  {
      return static_cast<TStaticData*>(Dll::Tls());
  }
  else
  {
      TInt error = KErrNone;
      TStaticData* sd = NULL;
      TRAP(error, {
        sd = new (ELeave) TStaticData;
      });
      if(error!=KErrNone){
        return (TStaticData*) SPyErr_SetFromSymbianOSErr(error);
      }

      TRAP(error, sd->telephony = CTelephony::NewL());

      if (error != KErrNone){
        delete sd;
        return (TStaticData*) SPyErr_SetFromSymbianOSErr(error);
      }
      error = Dll::SetTls(sd);
      if(error!=KErrNone){
        delete sd->telephony;
        delete sd;
        return (TStaticData*) SPyErr_SetFromSymbianOSErr(error);
      }

	  // DEBUG
      sd->iLog.Connect();
      sd->iLog.CreateLog(_L("PyS60"),_L("elocation_mod.txt"), EFileLoggingModeOverwrite);
      sd->iLog.Write(_L("elocation, open module"));
	  // !DEBUG
      PyThread_AtExit(location_mod_cleanup);
      return static_cast<TStaticData*>(Dll::Tls());
  }
}

extern "C" {
  void location_mod_cleanup()
  {
    TStaticData* sd = NULL;
    sd = GetTelephony();
	sd->iLog.Write(_L("elocation, close module"));

    if(sd!=NULL){
      delete sd->telephony;
      delete static_cast<TStaticData *>(Dll::Tls());
      Dll::SetTls(NULL);
    }
  }
}


extern "C" PyObject *
get_location(PyObject* /*self*/)
{
  TInt error = KErrNone;

  TStaticData* sd = NULL;
  sd = GetTelephony();
  if(sd==NULL){
    return NULL;
  }

  TRequestStatus status;
  CTelephony::TNetworkInfoV1 networkInfo;

  CTelephony::TNetworkInfoV1Pckg networkInfoPkg(networkInfo);

  Py_BEGIN_ALLOW_THREADS
  CAsyncCallHandler* asyncCallHandler = NULL;

  //debug
  sd->iLog.Write(_L("get_location"));

  TRAP(error,{
    asyncCallHandler = CAsyncCallHandler::NewL(*sd->telephony, sd->iLog);
    asyncCallHandler->MakeAsyncCall(networkInfoPkg,status);
  });
  
  delete asyncCallHandler;
  Py_END_ALLOW_THREADS
    
  
  if(error != KErrNone){
    return SPyErr_SetFromSymbianOSErr(error);
  }
  
  return Py_BuildValue("(u#u#iii)", 
         networkInfo.iCountryCode.Ptr(),networkInfo.iCountryCode.Length(), // string
         networkInfo.iNetworkId.Ptr(),networkInfo.iNetworkId.Length(), // string    
         networkInfo.iLocationAreaCode,
         networkInfo.iCellId,
         networkInfo.iAreaKnown);
}


extern "C" PyObject *
extended_gsm_location(PyObject* /*self*/)
{
  TInt error = KErrNone;
  
  TStaticData* sd = NULL;
  sd = GetTelephony();
  if(sd==NULL){
    return NULL;
  }
  //debug
  sd->iLog.Write(_L("extended_gsm_location"));

  TRequestStatus status;
  CTelephony::TNetworkInfoV1 networkInfo;
  CTelephony::TNetworkInfoV1Pckg networkInfoPkg(networkInfo);

  Py_BEGIN_ALLOW_THREADS
  CAsyncCallHandler* asyncCallHandler = NULL;

  TRAP(error,{
    asyncCallHandler = CAsyncCallHandler::NewL(*sd->telephony, sd->iLog);
    asyncCallHandler->MakeAsyncCall(networkInfoPkg,status);
  });

  //debug
  sd->iLog.WriteFormat(_L("AsyncCall return: %i"), error);

  delete asyncCallHandler;
  Py_END_ALLOW_THREADS

  if(error != KErrNone){
    return SPyErr_SetFromSymbianOSErr(error);
  }

  PyObject* NetInfoDict = PyDict_New();
  PyObject* key;
  PyObject* value;

  //debug
  sd->iLog.WriteFormat(_L("networkInfo.iMode: %i"), networkInfo.iMode);

  switch(networkInfo.iMode)
  {

  case CTelephony::ENetworkModeCdma95:
  case CTelephony::ENetworkModeCdma2000:
  case CTelephony::ENetworkModeTdcdma:
        key = Py_BuildValue("s", "BandInfo");
        switch(networkInfo.iBandInfo)
        {
            case CTelephony::E800BandA:
              value = Py_BuildValue("s", "800MHz Band A");
              break;
            case CTelephony::E800BandB:
              value = Py_BuildValue("s", "800MHz Band B");
              break;
            case CTelephony::E800BandC:
              value = Py_BuildValue("s", "800MHz Band C");
              break;
            case CTelephony::E1900BandA:
              value = Py_BuildValue("s", "1900MHz Band A");
              break;
            case CTelephony::E1900BandB:
              value = Py_BuildValue("s", "1900MHz Band B");
              break;
            case CTelephony::E1900BandC:
              value = Py_BuildValue("s", "1900MHz Band C");
              break;
            case CTelephony::E1900BandD:
              value = Py_BuildValue("s", "1900MHz Band D");
              break;
            case CTelephony::E1900BandE:
              value = Py_BuildValue("s", "1900MHz Band E");
              break;
            case CTelephony::E1900BandF:
              value = Py_BuildValue("s", "1900MHz Band F");
              break;
            case CTelephony::EBandUnknown:
            default:
              value = Py_BuildValue("s", "Unknown");
              break;
        }
        error = PyDict_SetItem(NetInfoDict, key, value);
        Py_DECREF(key);
        Py_DECREF(value);
        if (KErrNone != error)
        {
          Py_DECREF(NetInfoDict);
          Py_INCREF(Py_None);
          return Py_None;
        }
        // break; // NO BREAK HERE, next infos are also good for CDMA Networks
  case CTelephony::ENetworkModeAmps:
        key = Py_BuildValue("s", "CdmaSID");
        value = Py_BuildValue("u#", networkInfo.iCdmaSID.Ptr(), networkInfo.iCdmaSID.Length());
        error = PyDict_SetItem(NetInfoDict, key, value);
        Py_DECREF(key);
        Py_DECREF(value);
        if (KErrNone != error)
        {
          Py_DECREF(NetInfoDict);
          Py_INCREF(Py_None);
          return Py_None;
        }
        break;
  case CTelephony::ENetworkModeGsm:
  case CTelephony::ENetworkModeWcdma:
        key = Py_BuildValue("s", "NetworkAccessTechnology");
        switch(networkInfo.iAccess)
        {
        case CTelephony::ENetworkAccessGsm:
          value = Py_BuildValue("s", "GSM");
          break;
        case CTelephony::ENetworkAccessGsmCompact:
          value = Py_BuildValue("s", "GSM COMPACT");
          break;
        case CTelephony::ENetworkAccessUtran:
          value = Py_BuildValue("s", "UMTS");
          break;
        case CTelephony::ENetworkAccessUnknown:
        default:
          value = Py_BuildValue("s", "Unknown");
          break;
        }
        error = PyDict_SetItem(NetInfoDict, key, value);
        Py_DECREF(key);
        Py_DECREF(value);
        if (KErrNone != error)
        {
          Py_DECREF(NetInfoDict);
          Py_INCREF(Py_None);
          return Py_None;
        }
        key = Py_BuildValue("s", "ShortName");
        value = Py_BuildValue("u#", networkInfo.iShortName.Ptr(), networkInfo.iShortName.Length());
        error = PyDict_SetItem(NetInfoDict, key, value);
        Py_DECREF(key);
        Py_DECREF(value);
        if (KErrNone != error)
        {
          Py_DECREF(NetInfoDict);
          Py_INCREF(Py_None);
          return Py_None;
        }
        key = Py_BuildValue("s", "LongName");
        value = Py_BuildValue("u#", networkInfo.iLongName.Ptr(), networkInfo.iLongName.Length());
        error = PyDict_SetItem(NetInfoDict, key, value);
        Py_DECREF(key);
        Py_DECREF(value);
        if (KErrNone != error)
        {
          Py_DECREF(NetInfoDict);
          Py_INCREF(Py_None);
          return Py_None;
        }
        key = Py_BuildValue("s", "AreaKnown");
        value = Py_BuildValue("b", networkInfo.iAreaKnown ? 1 : 0);
        error = PyDict_SetItem(NetInfoDict, key, value);
        Py_DECREF(key);
        Py_DECREF(value);
        if (KErrNone != error)
        {
          Py_DECREF(NetInfoDict);
          return NULL;
        }
        key = Py_BuildValue("s", "AreaCode");
        if (networkInfo.iAreaKnown)
          value = Py_BuildValue("i", networkInfo.iLocationAreaCode);
        else
        {
          Py_INCREF(Py_None);
          value = Py_None;
        }
        error = PyDict_SetItem(NetInfoDict, key, value);
        Py_DECREF(key);
        Py_DECREF(value);
        if (KErrNone != error)
        {
          Py_DECREF(NetInfoDict);
          Py_INCREF(Py_None);
          return Py_None;
        }
        key = Py_BuildValue("s", "CellId");
        if (networkInfo.iAreaKnown)
          value = Py_BuildValue("i", networkInfo.iCellId);
        else {
          Py_INCREF(Py_None);
          value = Py_None;
        }
        error = PyDict_SetItem(NetInfoDict, key, value);
        Py_DECREF(key);
        Py_DECREF(value);
        if (KErrNone != error)
        {
          Py_DECREF(NetInfoDict);
          Py_INCREF(Py_None);
          return Py_None;
        }
        key = Py_BuildValue("s", "NetworkStatus");
        if (networkInfo.iAreaKnown)
        {
          switch(networkInfo.iStatus)
          {
          case CTelephony::ENetworkStatusAvailable:
            value = Py_BuildValue("s", "Available");
            break;
          case CTelephony::ENetworkStatusCurrent:
            value = Py_BuildValue("s", "Current");
            break;
          case CTelephony::ENetworkStatusForbidden:
            value = Py_BuildValue("s", "Forbidden");
            break;
          case CTelephony::ENetworkStatusUnknown:
          default:
            value = Py_BuildValue("s", "Unknown");
            break;
          }
        }
        else {
          Py_INCREF(Py_None);
          value = Py_None;
        }
        error = PyDict_SetItem(NetInfoDict, key, value);
        Py_DECREF(key);
        Py_DECREF(value);
        if (KErrNone != error)
        {
          Py_DECREF(NetInfoDict);
          Py_INCREF(Py_None);
          return Py_None;
        }
      break;
    default:
      break;
    }
    // All Networks
    key = Py_BuildValue("s", "NetworkMode");
    switch(networkInfo.iMode)
    {
    case CTelephony::ENetworkModeUnregistered:
      value = Py_BuildValue("s", "Unregistered");
      break;
    case CTelephony::ENetworkModeGsm:
      value = Py_BuildValue("s", "GSM/GPRS");
      break;
    case CTelephony::ENetworkModeAmps:
      value = Py_BuildValue("s", "AMPS");
      break;
    case CTelephony::ENetworkModeCdma95:
      value = Py_BuildValue("s", "IS-95");
      break;
    case CTelephony::ENetworkModeCdma2000:
      value = Py_BuildValue("s", "CDMA2000");
      break;
    case CTelephony::ENetworkModeWcdma:
      value = Py_BuildValue("s", "WCDMA");
      break;
    case CTelephony::ENetworkModeTdcdma:
      value = Py_BuildValue("s", "TD-CDMA");
      break;
    case CTelephony::ENetworkModeUnknown:
    default:
      value = Py_BuildValue("s", "Unknown");
      break;
    }
    error = PyDict_SetItem(NetInfoDict, key, value);
    Py_DECREF(key);
    Py_DECREF(value);
    if (KErrNone != error)
    {
      Py_DECREF(NetInfoDict);
      Py_INCREF(Py_None);
      return Py_None;
    }
    key = Py_BuildValue("s", "CountryCode");
    value = Py_BuildValue("u#", networkInfo.iCountryCode.Ptr(), networkInfo.iCountryCode.Length());
    error = PyDict_SetItem(NetInfoDict, key, value);
    Py_DECREF(key);
    Py_DECREF(value);
    if (KErrNone != error)
    {
      Py_DECREF(NetInfoDict);
      Py_INCREF(Py_None);
      return Py_None;
    }
    key = Py_BuildValue("s", "NetworkId"); // NID in CDMA / MNC in GSM
    value = Py_BuildValue("u#", networkInfo.iNetworkId.Ptr(), networkInfo.iNetworkId.Length());
    error = PyDict_SetItem(NetInfoDict, key, value);
    Py_DECREF(key);
    Py_DECREF(value);
    if (KErrNone != error)
    {
      Py_DECREF(NetInfoDict);
      Py_INCREF(Py_None);
      return Py_None;
    }
    key = Py_BuildValue("s", "DisplayTag");
    value = Py_BuildValue("u#", networkInfo.iDisplayTag.Ptr(), networkInfo.iDisplayTag.Length());
    error = PyDict_SetItem(NetInfoDict, key, value);
    Py_DECREF(key);
    Py_DECREF(value);
    if (KErrNone != error)
    {
      Py_DECREF(NetInfoDict);
      Py_INCREF(Py_None);
      return Py_None;
    }
  // debug
  sd->iLog.Write(_L("return NetInfoDict"));
  return NetInfoDict;
}


extern "C" PyObject *
get_registration_status(PyObject* /*self*/)
{
  TInt error = KErrNone;
  
  TStaticData* sd = NULL;
  sd = GetTelephony();
  if(sd==NULL){
    return NULL;
  }
  
  TRequestStatus status;
  CTelephony::TNetworkRegistrationV1 networkRegistration;
  CTelephony::TNetworkRegistrationV1Pckg networkRegistrationPckg(networkRegistration);

  Py_BEGIN_ALLOW_THREADS
  CAsyncCallHandler* asyncCallHandler = NULL;

  TRAP(error,{
    asyncCallHandler = CAsyncCallHandler::NewL(*sd->telephony, sd->iLog);
    asyncCallHandler->MakeAsyncCall_Reg(networkRegistrationPckg, status);
  });

  delete asyncCallHandler;
  Py_END_ALLOW_THREADS

  if(error != KErrNone)
    return SPyErr_SetFromSymbianOSErr(error);
	
  PyObject* value;
/*
enum TRegistrationStatus
*/
  switch (networkRegistration.iRegStatus) {
	case CTelephony::ENotRegisteredNoService:
		value = Py_BuildValue("s", "NotRegisteredNoService");
		//Not registered. The ME can not detect any other networks and is not currently searching a new operator to register to.
		break;
	case CTelephony::ENotRegisteredEmergencyOnly:
		value = Py_BuildValue("s", "NotRegisteredEmergencyOnly");
		//Not registered. The ME can detect other networks on which it is possible to make emergency calls only.
		break;
	case CTelephony::ENotRegisteredSearching:
		//Not registered, but the ME is currently searching a new operator to register to.
		value = Py_BuildValue("s", "NotRegisteredSearching");
		break;
	case CTelephony::ERegisteredBusy:
		//Registered, network busy.
		value = Py_BuildValue("s", "RegisteredBusy");
		break;
	case CTelephony::ERegisteredOnHomeNetwork:
		//Registered on home network.
		value = Py_BuildValue("s", "RegisteredOnHomeNetwork");
		break;
	case CTelephony::ERegistrationDenied:
		value = Py_BuildValue("s", "RegistrationDenied");
		//"Registration denied"
		break;
	case CTelephony::ERegisteredRoaming:
		value = Py_BuildValue("s", "RegisteredRoaming");
		//"Registered, roaming"
		break;
	case CTelephony::ERegistrationUnknown:
	default:
		value = Py_BuildValue("s", "RegistrationUnknown");
		break;
  }
  return value;
}



extern "C" PyObject *
getIMSI(PyObject* /*self*/)
{
  TInt error = KErrNone;
  PyObject* value = NULL;

  TStaticData* sd = NULL;
  sd = GetTelephony();
  if(sd == NULL){
    return NULL;
  }

// TSubscriberIdV1()
// TBuf< KIMSISize > iSubscriberId

  TRequestStatus status;
  CTelephony::TSubscriberIdV1 subscriberId;
  CTelephony::TSubscriberIdV1Pckg subscriberIdPckg(subscriberId);

  Py_BEGIN_ALLOW_THREADS
  CAsyncCallHandler* asyncCallHandler = NULL;

  //debug
  sd->iLog.Write(_L("getIMSI"));

  TRAP(error,{
	sd->iLog.Write(_L("New AsyncCallHandler"));
    asyncCallHandler = CAsyncCallHandler::NewL(*sd->telephony, sd->iLog);
    sd->iLog.Write(_L("MakeAsyncCall"));
	asyncCallHandler->MakeAsyncCall_IMSI(subscriberIdPckg, status);
  });
  sd->iLog.WriteFormat(_L("Return from AsyncCall, status(%i)"), status.Int());
  delete asyncCallHandler;
  Py_END_ALLOW_THREADS
    
  sd->iLog.Write(_L("Test return value from AsyncCall"));
  if(error != KErrNone){
	sd->iLog.WriteFormat(_L(" AsyncCall returned something else than KErrNone (%i)"), error);
	return SPyErr_SetFromSymbianOSErr(error);
  }
  sd->iLog.Write(_L("  AsyncCall returned KErrNone"));
  
  if (subscriberId.iSubscriberId.Ptr() != NULL) {
    sd->iLog.Write(_L("Building return Value"));
    value = Py_BuildValue("u#", 
	  subscriberId.iSubscriberId.Ptr(), subscriberId.iSubscriberId.Length() // string
    );
  }
  else {
    sd->iLog.Write(_L("Building return NULL Value"));
    value = Py_BuildValue("s", "None");
  }    
  sd->iLog.Write(_L("returning the value"));
  return value;
}

#endif /* EKA2 */

extern "C" {

  static const PyMethodDef location_methods[] = {
    {"gsm_location", (PyCFunction)get_location, METH_NOARGS, NULL},
    {"extended_gsm_location", (PyCFunction)extended_gsm_location, METH_NOARGS, NULL},
    {"get_registration_status", (PyCFunction)get_registration_status, METH_NOARGS, NULL},
    {"get_imsi", (PyCFunction)getIMSI, METH_NOARGS, NULL},
	{"version", (PyCFunction)getVersion, METH_NOARGS, NULL},
    {NULL,              NULL}           /* sentinel */
  };

  DL_EXPORT(void) initlocation(void)
  {
    Py_InitModule("_elocation", (PyMethodDef*)location_methods);
    return;
  }
} /* extern "C" */

#ifndef EKA2
GLDEF_C TInt E32Dll(TDllReason)
{
  return KErrNone;
}
#endif
