/**
 * ====================================================================
 *  gps_location.cpp
 *
 *  Python API to gps location information.
 *
 *  Copyright (c) 2007 Christophe Berger <christophe.berger@lip6.fr>
 *
 *  Based on code by Nokia Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

#include <e32std.h>
#include <lbs.h>
#include <LbsSatellite.h>
#include <flogger.h>

#include "Python.h"
#include "symbian_python_ext_util.h"

//#include "asynccallhandler.h"
#include "LBSPositionRequestor.h"
#include "LBSPositionListener.h"	
 
//_LIT(KRequestor, "Python");
//_LIT(KUnknown, "Unknown");

const TInt KSecond = 1000000;
const TInt KUpdateInterval = 3*KSecond; //Update interval
const TInt KUpdateTimeOut = 2*KSecond; //Update time out
const TInt KMaxAge = 900000; //0.9*KSecond;

extern "C"
void gps_location_mod_cleanup();



class TStaticPositionData: public MLbsPositionListener
    {
  public:
    // Position server
    //RPositionServer iPosServer;
    // Positioner
    //RPositioner iPositioner;
    // DEBUG
    RFileLogger iLog;
	
	void PositionInfoUpdatedL(TPositionInfo& aPosInfo, const TDesC& aModulename)
	{
	  iPosInfo = aPosInfo;
	  iModName = aModulename;
	  iLog.Write(_L("Update"));
	  
	  TPosition pos;
  	  aPosInfo.GetPosition(pos);
	  
	  iLog.WriteFormat(_L("Position: %f ; %f"), pos.Latitude(), pos.Longitude());
		
	  
    }
	
	void ShowErrorL(const TDesC& aErrorString)
	{
	  iLog.WriteFormat(_L("Error: %S"), aErrorString.Ptr());
	}
        
    void ShowMessageL(const TDesC& aMessage)
	{
	  iLog.WriteFormat(_L("Msg: %S"), aMessage.Ptr());
	}
	
    virtual ~TStaticPositionData()
    {
	    delete iPositionRequestor;
		iPositionRequestor = NULL;
	}
  //protected:
  CLbsPositionRequestor* iPositionRequestor;
  TPositionInfo iPosInfo;
  TBuf<KPositionMaxModuleName> iModName;
};



TStaticPositionData* GetGpsLocation()
{
  if (Dll::Tls())
  {
      return static_cast<TStaticPositionData*>(Dll::Tls());
  }
  else
  {
      TInt error = KErrNone;
      TStaticPositionData* sd = NULL;
      TRAP(error, {
        sd = new (ELeave) TStaticPositionData;
      });
      if(error != KErrNone){
        return (TStaticPositionData*) SPyErr_SetFromSymbianOSErr(error);
      }

      // DEBUG
      sd->iLog.Connect();
      sd->iLog.CreateLog(_L("PyS60"),_L("gpsloc_mod.txt"), EFileLoggingModeOverwrite);
      //sd->iLog.Write(_L("Open logfile"));
      // !DEBUG

      TRAP(error, sd->iPositionRequestor = CLbsPositionRequestor::NewL(*sd));
      if (error != KErrNone)
      {
	    sd->iLog.Write(_L(" Error new positionrequestor"));
        delete sd;
        return (TStaticPositionData*) SPyErr_SetFromSymbianOSErr(error);
      }
	  
	
      error = Dll::SetTls(sd);
      if(error != KErrNone)
      {
        // Cancel requests to iPositioner ?!?
        delete sd->iPositionRequestor;
		delete sd;
        return (TStaticPositionData*) SPyErr_SetFromSymbianOSErr(error);
      }

      PyThread_AtExit(gps_location_mod_cleanup);
      return static_cast<TStaticPositionData*>(Dll::Tls());
  }
}

extern "C" {
  void gps_location_mod_cleanup()
  {
    TStaticPositionData* sd = NULL;
    sd = GetGpsLocation();

    if(sd != NULL)
    {
      sd->iLog.Write(_L("mod_cleanup"));
      delete sd;
      Dll::SetTls(NULL);
    }
  }
}


extern "C" PyObject *
module_name(PyObject* /*self*/)
{
    TStaticPositionData*    sd = NULL;
    sd = GetGpsLocation();
    if(sd != NULL)
    {
        return Py_BuildValue("u#", sd->iModName.Ptr(), sd->iModName.Length());
    }
    else
    {
      return NULL;
    }
}


extern "C" PyObject *
SetUpdateOptions(PyObject* /*self*/, PyObject* args)
{
    TStaticPositionData*    sd = NULL;
	TInt					UpdateIntervalMicroSeconds = 1000000;
	TInt					UpdateTimeOutMicroSeconds = 15000000;
	TInt					MaxAgeMicroSeconds = 500000;
	TBool					AcceptPartialUpdates = EFalse;
		
    sd = GetGpsLocation();
    if(sd != NULL && sd->iPositionRequestor)
    {
        if (!PyArg_ParseTuple(args, "iiib", 
		&UpdateIntervalMicroSeconds,
		&UpdateTimeOutMicroSeconds,
		&MaxAgeMicroSeconds,
		&AcceptPartialUpdates))
		{
			  PyErr_SetString(PyExc_ValueError,  "Arguments error");
	          return NULL;
	    }
		TInt error = sd->iPositionRequestor->SetUpdateOptions(UpdateIntervalMicroSeconds,
				UpdateTimeOutMicroSeconds,
				MaxAgeMicroSeconds,
				AcceptPartialUpdates);

		return Py_BuildValue("(i)", error);
    }
    else
    {
	  PyErr_SetString(PyExc_ValueError,  "Static Data error");
      return NULL;
    }
}


		
		


extern "C" PyObject *
position(PyObject* /*self*/, PyObject* /*args*/)
{
  TStaticPositionData*  sd = NULL;
  TInt error = KErrNone;
  
  sd = GetGpsLocation();
  if(sd == NULL){
    return NULL;
  }

//  if (!PyArg_ParseTuple(args, "|b", &ask_device))
//    return NULL;

  TPosition position;
  sd->iPosInfo.GetPosition(position);

  sd->iLog.WriteFormat(_L("Position: %f  %f  %f  %f  %f"), position.Latitude(),
         position.Longitude(),
         position.Altitude(),
         position.VerticalAccuracy(),
         position.HorizontalAccuracy());

	PyObject* PositionDict = PyDict_New();
	PyObject* key;
	PyObject* value;
// Longitude
	key = Py_BuildValue("s", "Longitude");
	if (not isnan(position.Longitude()))
	  value = Py_BuildValue("f", position.Longitude());
	else {
	  Py_INCREF(Py_None);
	  value = Py_None;
    }
	error = PyDict_SetItem(PositionDict, key, value);
    Py_DECREF(key);
    Py_DECREF(value);
    if (KErrNone != error)
	{
      Py_DECREF(PositionDict);
      return NULL;
    }
// Latitude
	key = Py_BuildValue("s", "Latitude");
	if (not isnan(position.Latitude()))
	  value = Py_BuildValue("f", position.Latitude());
	else {
	  Py_INCREF(Py_None);
	  value = Py_None;
    }
	error = PyDict_SetItem(PositionDict, key, value);
    Py_DECREF(key);
    Py_DECREF(value);
    if (KErrNone != error)
	{
      Py_DECREF(PositionDict);
      return NULL;
    }
// Altitude
	key = Py_BuildValue("s", "Altitude");
	if (not isnan(position.Altitude()))
	  value = Py_BuildValue("f", position.Altitude());
	else {
	  Py_INCREF(Py_None);
	  value = Py_None;
    }
	error = PyDict_SetItem(PositionDict, key, value);
    Py_DECREF(key);
    Py_DECREF(value);
    if (KErrNone != error)
	{
      Py_DECREF(PositionDict);
      return NULL;
    }
// VerticalAccuracy
	key = Py_BuildValue("s", "VerticalAccuracy");
	if (not isnan(position.VerticalAccuracy()))
	  value = Py_BuildValue("f", position.VerticalAccuracy());
	else {
	  Py_INCREF(Py_None);
	  value = Py_None;
    }
	error = PyDict_SetItem(PositionDict, key, value);
    Py_DECREF(key);
    Py_DECREF(value);
    if (KErrNone != error)
	{
      Py_DECREF(PositionDict);
      return NULL;
    }
// Horizontal Accuracy
	key = Py_BuildValue("s", "HorizontalAccuracy");
	if (not isnan(position.HorizontalAccuracy()))
	  value = Py_BuildValue("f", position.HorizontalAccuracy());
	else {
	  Py_INCREF(Py_None);
	  value = Py_None;
    }
	error = PyDict_SetItem(PositionDict, key, value);
    Py_DECREF(key);
    Py_DECREF(value);
    if (KErrNone != error)
	{
      Py_DECREF(PositionDict);
      return NULL;
    }

	return PositionDict;

  // return Py_BuildValue("(fffff)",
         // position.Latitude(),
         // position.Longitude(),
         // position.Altitude(),
         // position.VerticalAccuracy(),
         // position.HorizontalAccuracy());
}


extern "C" PyObject *
position_timestamp(PyObject* /*self*/, PyObject* /*args*/)
{
  TStaticPositionData*  sd = NULL;
  TPosition             position;
  TDateTime             dateTime;
  
  sd = GetGpsLocation();
  if(sd == NULL){
    return NULL;
  }

  sd->iPosInfo.GetPosition(position);
  dateTime = position.Time().DateTime();

  return Py_BuildValue("(iiiiiii)",
         dateTime.Year(),
         dateTime.Month(),
         dateTime.Day() + 1,
         dateTime.Hour(),
         dateTime.Minute(),
         dateTime.Second(),
         dateTime.MicroSecond());
}




/* These functions come from Nokia's official positioning module */
/*
 * Convert epoch to TReal.
 */
static TReal epochToReal()
{
  _LIT(KEpoch, "19700000:");
  TTime epochTime;
  epochTime.Set(KEpoch);
  #ifndef EKA2
  return epochTime.Int64().GetTReal();
  #else
  return TReal64(epochTime.Int64());
  #endif
}


/*
 * TTime to python real.
 */
static TReal timeToReal(const TTime& time)
{
  TLocale locale;
  #ifndef EKA2
  return (((time.Int64().GetTReal()-epochToReal())/
          (1000.0*1000.0))-
          TInt64(locale.UniversalTimeOffset().Int()).GetTReal());
  #else
  return (((TReal64(time.Int64())-epochToReal())/(1000.0*1000.0)) 
        - TInt64(TReal64(locale.UniversalTimeOffset().Int())));
  #endif
}
/* End functions from Nokia */


extern "C" PyObject *
position_age(PyObject* /*self*/, PyObject* /*args*/)
{
  TStaticPositionData*  sd = NULL;
  TPosition             position;
  //TTimeIntervalSeconds  SecondsInterval;
  TTime                 time;

  sd = GetGpsLocation();
  if(sd == NULL){
    return NULL;
  }

  sd->iPosInfo.GetPosition(position); 
  time.HomeTime();
  
  //position.Time().SecondsFrom(time, SecondsInterval);  

  TReal posAge = timeToReal(time) - timeToReal(position.Time());
  
  return Py_BuildValue("f", posAge); //SecondsInterval.Int());
}


extern "C" PyObject *
phone_time_list(PyObject* /*self*/, PyObject* /*args*/)
{
  TTime                 time;
  TDateTime             dateTime;

  time.HomeTime();
  dateTime = time.DateTime();

  return Py_BuildValue("(iiiiiii)",
         dateTime.Year(),
         dateTime.Month(),
         dateTime.Day() + 1,
         dateTime.Hour(),
         dateTime.Minute(),
         dateTime.Second(),
         dateTime.MicroSecond());
}

extern "C" PyObject *
phone_time_real(PyObject* /*self*/, PyObject* /*args*/)
{
  TTime                 time;
  
  time.HomeTime();
  
  return Py_BuildValue("f",
          timeToReal(time));
}


extern "C" {

  static const PyMethodDef
  location_methods[] = {
    {"position_age",        (PyCFunction)position_age,       METH_NOARGS, NULL},
	{"position",            (PyCFunction)position,           METH_NOARGS, NULL},
    {"position_timestamp",  (PyCFunction)position_timestamp, METH_NOARGS, NULL},
    {"phone_time",          (PyCFunction)phone_time_list,    METH_NOARGS, NULL},
	{"phone_time_real",     (PyCFunction)phone_time_real,    METH_NOARGS, NULL},
    {"module_name",         (PyCFunction)module_name,        METH_NOARGS, NULL},
	{"set_update_options",  (PyCFunction)SetUpdateOptions,   METH_VARARGS, NULL},
    {NULL,            NULL} /* sentinel */
  };

  DL_EXPORT(void)
  initlocation(void)   // Here : this symbol was defined in some frz files...
  {
    PyObject *m;

    m = Py_InitModule("_gps_location", (PyMethodDef*)location_methods);
    return;
  }
} /* extern "C" */


