/**
 * ====================================================================
 *  wlantoolsmodule.cpp
 *
 *  Python API to Wlan Connection information.
 *
 *  Copyright (c) 2007 Christophe Berger <christophe.berger@lip6.fr>
 * Copyright (c) 2008 Christophe Berger <chris@berger.cx>
 *
 *  Based on code by Nokia Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */
#include <e32std.h>
#include <wlanscaninfo.h>
#include <wlanmgmtclient.h>
//#include <flogger.h>
#include <rconnmon.h>

#include "Python.h"
#include "symbian_python_ext_util.h"

#include "asynccallhandler.h"

extern "C"

void _mod_cleanup();


struct TStaticData {

    // DEBUG
  //RFileLogger iLog;
  CWlanScanInfo* iScanInfo;
  CWlanMgmtClient* iScan;
  RConnectionMonitor* iMonitor;
  TRequestStatus iStatus;
};



TStaticData* GetStaticData()
{
  if (Dll::Tls())
  {
      return static_cast<TStaticData*>(Dll::Tls());
  }
  else
  {
    TInt error = KErrNone;
    TStaticData* sd = NULL;
    TRAP(error, {
      sd = new (ELeave) TStaticData;
    });
    if(error != KErrNone){
      return (TStaticData*) SPyErr_SetFromSymbianOSErr(error);
    }
    // DEBUG
    //sd->iLog.Connect();
    //sd->iLog.CreateLog(_L("PyS60"),_L("wlantools_mod.txt"), EFileLoggingModeOverwrite);
    //sd->iLog.Write(_L("New instance"));
    // !DEBUG
    sd->iMonitor = NULL;
  
    TRAP(error, sd->iScan = CWlanMgmtClient::NewL());
    if (error != KErrNone){
      return (TStaticData*) SPyErr_SetFromSymbianOSErr(error);    
    }

    TRAP(error, sd->iScanInfo = CWlanScanInfo::NewL());
    if (error != KErrNone){
      return (TStaticData*) SPyErr_SetFromSymbianOSErr(error);    
    } 
  
    error = Dll::SetTls(sd);
    if(error != KErrNone)
    {
        return (TStaticData*) SPyErr_SetFromSymbianOSErr(error);
    }

    PyThread_AtExit(_mod_cleanup);
    return static_cast<TStaticData*>(Dll::Tls());
  }
}



extern "C" {
  void _mod_cleanup()
  {
    TStaticData* sd = NULL;
    sd = GetStaticData();
  
    if(sd != NULL)
    {
    //sd->iLog.Write(_L("Cleanup, freeing static data"));
    //  sd->iLog.CloseLog();
    //  sd->iLog.Close();
    if (sd->iScan)
    {
      delete sd->iScan;
      sd->iScan = NULL;
    }  
      if (sd->iScanInfo)
    {
        delete sd->iScanInfo;
        sd->iScanInfo = NULL;
    }
      if (sd->iMonitor) {
        delete sd->iMonitor;
        sd->iMonitor = NULL;
    }
    delete static_cast<TStaticData*>(Dll::Tls());
      Dll::SetTls(NULL);
    }
  }
}



TInt wlan_count()
{
    TStaticData* sd = NULL;
    sd = GetStaticData();
    if(sd == NULL)
    {
    PyErr_SetString(PyExc_ValueError,  "Can't get a valid pointer on the static data");
    return -1;
  }

  TInt count = 0;
  for(sd->iScanInfo->First(); !sd->iScanInfo->IsDone(); sd->iScanInfo->Next())
    ++count;
    sd->iScanInfo->First();
  
  //sd->iLog.WriteFormat(_L(" Number of wlans discovered: %d"), count);
  
  return count;
}



extern "C" PyObject*
get_wlan_item_dict(TStaticData* sd, CWlanScanInfo* aItem, TBool getIE)
{
  if (!aItem or aItem->IsDone())
    return NULL;

  const TUint8 KWlan802Dot11SsidIE = 0; // Information Element ID for SSID as specified in 802.11.
  const TUint8 KWlan802Dot11SupportedRatesIE = 1; // Information Element ID for Supported Rates Parameter Set as specified in IEEE802.11-1999, A.3.2.2
  const TUint8 KWlan802Dot11DsIE = 3; // Information Element ID for DS Parameter Set as specified in IEEE802.11-1999, A.3.2.4
  TBuf<255> info; // 48 for SSID but variable for Information Elements, 255 is max size of data (size on 1 octet)
  TInt error = KErrNone;
  PyObject* WlanInfoDict = PyDict_New();
  PyObject* key;
  PyObject* value;
  TWlanBssid aBssid;
  TUint8 ieLen(0);
  TUint8 aIE(0);
  const TUint8* ieData(NULL);

// BSSID
  aItem->Bssid(aBssid);
  info.Zero();
  for (TUint i = 0; i < KWlanMaxBssidLength; ++i)
  {
    info.AppendFormat(_L("%02X"), aBssid[i]);
    if (i < KWlanMaxBssidLength - 1)
      info.Append(_L(":"));
  }
  key = Py_BuildValue("s", "BSSID");
  value = Py_BuildValue("u#", info.Ptr(), info.Length());
  error = PyDict_SetItem(WlanInfoDict, key, value);
  Py_DECREF(key);
  Py_DECREF(value);
  if (KErrNone != error)
  {
    Py_DECREF(WlanInfoDict);
    return NULL;
  }

// Connection mode
  TUint16 capa = 0;
  TUint16 mask = 0x2; // bit number 1 of the Capability INformation Field (See IEEE 802.11 Handbook)
  capa = aItem->Capability();
  key = Py_BuildValue("s", "ConnectionMode");
  if (capa & mask)
    value = Py_BuildValue("s", "Adhoc");
  else
    value = Py_BuildValue("s", "Infrastructure");
  error = PyDict_SetItem(WlanInfoDict, key, value);
  Py_DECREF(key);
  Py_DECREF(value);
  if (KErrNone != error)
  {
    Py_DECREF(WlanInfoDict);
    return NULL;
  }
  key = Py_BuildValue("s", "Capability");
  value = Py_BuildValue("i", capa);
  error = PyDict_SetItem(WlanInfoDict, key, value);
  Py_DECREF(key);
  Py_DECREF(value);
  if (KErrNone != error)
  {
    Py_DECREF(WlanInfoDict);
    return NULL;
  }
  // SSID
  //TUint8 ieLen(0);
  //TUint8 const* ieData(NULL);
  error = aItem->InformationElement(KWlan802Dot11SsidIE, ieLen, &ieData);
  if (KErrNone == error)
  {
    info.Zero();
    for (int i = 0; i < 32 and i < ieLen; ++i)
      info.Append(ieData[i]); // hand made netName.Copy(ieData);
  }
  key = Py_BuildValue("s", "SSID");
  value = Py_BuildValue("u#", info.Ptr(), info.Length());
  error = PyDict_SetItem(WlanInfoDict, key, value);
  Py_DECREF(key);
  Py_DECREF(value);
  if (KErrNone != error)
  {
    Py_DECREF(WlanInfoDict);
    return NULL;
  }
// RxLevel
  key = Py_BuildValue("s", "RxLevel");
  value = Py_BuildValue("i", -aItem->RXLevel());
  error = PyDict_SetItem(WlanInfoDict, key, value);
  Py_DECREF(key);
  Py_DECREF(value);
  if (KErrNone != error)
  {
    Py_DECREF(WlanInfoDict);
    return NULL;
  }
// BeaconInterval
  key = Py_BuildValue("s", "BeaconInterval");
  value = Py_BuildValue("i", aItem->BeaconInterval());
  error = PyDict_SetItem(WlanInfoDict, key, value);
  Py_DECREF(key);
  Py_DECREF(value);
  if (KErrNone != error)
  {
    Py_DECREF(WlanInfoDict);
    return NULL;
  }
//SecurityMode
  key = Py_BuildValue("s", "SecurityMode");
  TWlanConnectionSecurityMode sm = aItem->SecurityMode();
  switch (sm) {
    case EWlanConnectionSecurityOpen:
      value = Py_BuildValue("s", "Open");
      break;
    case EWlanConnectionSecurityWep:
      value = Py_BuildValue("s", "Wep");
      break;
    case EWlanConnectionSecurity802d1x:
      value = Py_BuildValue("s", "802.1x");
      break;
    case EWlanConnectionSecurityWpa:
      value = Py_BuildValue("s", "Wpa");
      break;
    case EWlanConnectionSecurityWpaPsk:
      value = Py_BuildValue("s", "WpaPsk");
      break;
    default:
      value = Py_BuildValue("s", "Unidentified");
      break;
  }
  error = PyDict_SetItem(WlanInfoDict, key, value);
  Py_DECREF(key);
  Py_DECREF(value);
  if (KErrNone != error)
  {
    Py_DECREF(WlanInfoDict);
    return NULL;
  }

// Additional data fetched upon request
  // DS IE
   error = aItem->InformationElement(KWlan802Dot11DsIE, ieLen, &ieData);
   if (KErrNone == error && ieLen > 0)
   {
     key = Py_BuildValue("s", "Channel");
     value = Py_BuildValue("i", ieData[0]);
     error = PyDict_SetItem(WlanInfoDict, key, value);
     Py_DECREF(key);
     Py_DECREF(value);
     if (KErrNone != error)
     {
       Py_DECREF(WlanInfoDict);
       return NULL;
     }
   }
  // SupportedRates IE
   error = aItem->InformationElement(KWlan802Dot11SupportedRatesIE, ieLen, &ieData);
   if (KErrNone == error && ieLen > 0)
   {
     info.Zero();
     TUint end = ieLen < 255 ? ieLen : 255;
     for (TUint i = 0; i < end; ++i)
     {
       info.AppendFormat(_L("%02X"), ieData[i]);
       //if (i < end - 1)
       //  info.Append(_L(":"));
     }
     key = Py_BuildValue("s", "SupportedRates");
     value = Py_BuildValue("u#", info.Ptr(), info.Length());
     error = PyDict_SetItem(WlanInfoDict, key, value);
     Py_DECREF(key);
     Py_DECREF(value);
     if (KErrNone != error)
     {
       Py_DECREF(WlanInfoDict);
       return NULL;
     }
   }
  if (getIE) {
    PyObject* WlanIEDict = PyDict_New();
    for (error = aItem->FirstIE(aIE, ieLen, &ieData);
         ieLen > 0 && KErrNone == error; // Maximum security, leave if any error/inconsticency in data retrieved
         error = aItem->NextIE(aIE, ieLen, &ieData))
    {
      if (aIE == 1 || aIE == 3 || aIE == 0)
        continue; // Do not retrieve IE already in Dict!
      key = Py_BuildValue("i", aIE);
      info.Zero();
      TUint end = ieLen < 255 ? ieLen : 255;
      for (TUint i = 0; i < end; ++i)
        info.AppendFormat(_L("%02X"), ieData[i]);
      value = Py_BuildValue("u#", info.Ptr(), info.Length()); //ieData, ieLen);
      error = PyDict_SetItem(WlanIEDict, key, value);
      Py_DECREF(key);
      Py_DECREF(value);
      if (KErrNone != error)
      {
        Py_DECREF(WlanInfoDict);
        Py_DECREF(WlanIEDict);
        return NULL;
      }
    }
    key = Py_BuildValue("s", "InformationElements");
    error = PyDict_SetItem(WlanInfoDict, key, WlanIEDict);
    Py_DECREF(key);
    Py_DECREF(WlanIEDict);
    if (KErrNone != error)
    {
      Py_DECREF(WlanInfoDict);
      Py_DECREF(WlanIEDict);
      return NULL;
    }
  }
  return WlanInfoDict;
}

extern "C" PyObject*
WlanScan_withIE(PyObject* /*self*/, PyObject* args)
{
    TBool getIE = EFalse;
    TStaticData* sd = NULL;
    sd = GetStaticData();
    if(sd == NULL)
    {
      PyErr_SetString(PyExc_ValueError,  "Can't get a valid pointer on the static data");
      return NULL;
  }

  if (!args)
    getIE = EFalse;
  else
    if (!PyArg_ParseTuple(args, "b", &getIE))
      getIE = EFalse;

  sd->iScan->GetScanResults(*(sd->iScanInfo));
  TInt count = wlan_count();
  PyObject* WlanList = PyList_New(count);

  TInt i = 0;
  for(i = 0, sd->iScanInfo->First();
    !sd->iScanInfo->IsDone() and i < count;
    sd->iScanInfo->Next(), ++i)
  {
      PyObject* item = get_wlan_item_dict(sd, sd->iScanInfo, getIE); // FIXME error handling
      PyList_SetItem(WlanList, i, item); // FIXME error handling
  }
  return WlanList;
}




extern "C" {

  static const PyMethodDef
 wlantools_methods[] = {
    {"scan", (PyCFunction)WlanScan_withIE, METH_VARARGS, NULL},
    {NULL,   NULL} /* sentinel */
  };

  DL_EXPORT(void)
  initwlantools(void)   // Here : this symbol was defined in some frz files...
  {
    PyObject *m;

    m = Py_InitModule("_wlantools", (PyMethodDef*)wlantools_methods);
    return;
  }
} /* extern "C" */


